@@ -1,28 +1,28 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*-
+ * gtksourceengine.h - Abstract base class for highlighting engines
+ * This file is part of GtkSourceView
  *
- *  gtksourceengine.h - Abstract base class for highlighting engines
+ * Copyright (C) 2003 - Gustavo Giráldez
  *
- *  Copyright (C) 2003 - Gustavo Giráldez
+ * GtkSourceView is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Library General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * GtkSourceView is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef __GTK_SOURCE_ENGINE_H__
 #define __GTK_SOURCE_ENGINE_H__
 
-#include <gtk/gtktextbuffer.h>
+#include <gtk/gtk.h>
 #include <gtksourceview/gtksourcestylescheme.h>
 
 G_BEGIN_DECLS
@@ -34,12 +34,12 @@ G_BEGIN_DECLS
 #define GTK_IS_SOURCE_ENGINE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_SOURCE_ENGINE))
 #define GTK_SOURCE_ENGINE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_SOURCE_ENGINE, GtkSourceEngineClass))
 
-typedef struct _GtkSourceEngine	GtkSourceEngine;
+typedef struct _GtkSourceEngine		GtkSourceEngine;
 typedef struct _GtkSourceEngineClass	GtkSourceEngineClass;
 
 struct _GtkSourceEngine
 {
-	GObject g_object;
+	GObject parent_instance;
 };
 
 struct _GtkSourceEngineClass
@@ -63,6 +63,10 @@ struct _GtkSourceEngineClass
 
 	void     (* set_style_scheme) (GtkSourceEngine      *engine,
 				       GtkSourceStyleScheme *scheme);
+
+	GtkTextTag *(* get_context_class_tag)
+				      (GtkSourceEngine      *engine,
+				       const gchar          *context_class);
 };
 
 GType       _gtk_source_engine_get_type		(void) G_GNUC_CONST;
@@ -82,6 +86,10 @@ void        _gtk_source_engine_update_highlight	(GtkSourceEngine      *engine,
 void        _gtk_source_engine_set_style_scheme	(GtkSourceEngine      *engine,
 						 GtkSourceStyleScheme *scheme);
 
+GtkTextTag *_gtk_source_engine_get_context_class_tag
+						 (GtkSourceEngine     *engine,
+						  const gchar         *context_class);
+
 G_END_DECLS
 
 #endif /* __GTK_SOURCE_ENGINE_H__ */
