@@ -197,8 +197,8 @@ gtk_source_language_manager_new (void)
  *
  * Returns the default #GtkSourceLanguageManager instance.
  *
- * Returns: (transfer-none): a #GtkSourceLanguageManager. Return value is owned
- * by GtkSourceView library and must not be unref'ed.
+ * Returns: (transfer none): a #GtkSourceLanguageManager.
+ * Return value is owned by GtkSourceView library and must not be unref'ed.
  */
 GtkSourceLanguageManager *
 gtk_source_language_manager_get_default (void)
@@ -271,7 +271,7 @@ gtk_source_language_manager_set_search_path (GtkSourceLanguageManager *lm,
  *
  * Gets the list directories where @lm looks for language files.
  *
- * Returns: (array zero-terminated) (transfer none): %NULL-terminated array
+ * Returns: (array zero-terminated=1) (transfer none): %NULL-terminated array
  * containg a list of language files directories.
  * The array is owned by @lm and must not be modified.
  */
@@ -389,9 +389,10 @@ ensure_languages (GtkSourceLanguageManager *lm)
  *
  * Returns the ids of the available languages.
  *
- * Returns: a %NULL-terminated array of string containing the ids of the
- * available languages or %NULL if no language is available. The array
- * is owned by @lm and must not be modified.
+ * Returns: (transfer none): a %NULL-terminated array of string
+ * containing the ids of the available languages or %NULL if
+ * no language is available.
+ * The array is owned by @lm and must not be modified.
  */
 G_CONST_RETURN gchar* G_CONST_RETURN *
 gtk_source_language_manager_get_language_ids (GtkSourceLanguageManager *lm)
@@ -625,9 +626,8 @@ gtk_source_language_manager_guess_language (GtkSourceLanguageManager *lm,
 	GSList *langs = NULL;
 
 	g_return_val_if_fail (GTK_IS_SOURCE_LANGUAGE_MANAGER (lm), NULL);
-	g_return_val_if_fail (filename != NULL || content_type != NULL, NULL);
-	g_return_val_if_fail ((filename == NULL || *filename != 0) &&
-			      (content_type == NULL || *content_type != 0), NULL);
+	g_return_val_if_fail ((filename != NULL && *filename != '\0') ||
+	                      (content_type != NULL && *content_type != '\0'), NULL);
 
 	ensure_languages (lm);
 
@@ -638,7 +638,7 @@ gtk_source_language_manager_guess_language (GtkSourceLanguageManager *lm,
 	  - no glob matches
 	*/
 
-	if (filename != NULL)
+	if (filename != NULL && *filename != '\0')
 		langs = pick_langs_for_filename (lm, filename);
 
 	if (langs != NULL)
